@@ -463,8 +463,9 @@ linc_connection_read (LINCConnection *cnx,
 			}
 
 		} else if (n == 0) {
-			if (cnx->options & LINC_CONNECTION_NONBLOCKING)
-				return LINC_IO_FATAL_ERROR;
+			d_printf ("we got EOF on fd %d\n", cnx->priv->fd);
+			linc_connection_state_changed (cnx, LINC_DISCONNECTED);
+			return LINC_IO_FATAL_ERROR;
 		} else {
 			buf += n;
 			len -= n;
@@ -477,18 +478,61 @@ linc_connection_read (LINCConnection *cnx,
 	return bytes_read;
 }
 
-static LINCIOStatus
-write_data (LINCConnection *cnx, QueuedWrite *qw, gboolean queue_write)
+/* Determine the maximum size of the iovec vector */
+
+#if defined (UIO_MAXIOV) /* Glibc */
+# define LINC_IOV_MAX (UIO_MAXIOV)
+#elif defined (MAXIOV) /* HPUX */
+# define LINC_IOV_MAX (MAXIOV)
+#elif defined (IOV_MAX) /* AIX */
+# define LINC_IOV_MAX (IOV_MAX)
+#elif defined (_SC_IOV_MAX) /* SGI */
+# define LINC_IOV_MAX_INIT (sysconf (_SC_IOV_MAX))
+#else /* Safe Guess */
+# define LINC_IOV_MAX 16
+#endif
+
+/* If the value requires initialization, define the function here */
+#if defined (LINC_IOV_MAX_INIT)
+# define LINC_IOV_MAX linc_iov_max
+  static guint linc_iov_max = 0;
+  static inline void
+  linc_iov_max_init () 
+  {
+    if (linc_iov_max == 0)
+      {
+        gint max;
+        G_LOCK_DEFINE_STATIC (linc_iov_max);
+        G_LOCK (linc_iov_max);
+        if (linc_iov_max == 0)
+          {
+            max = LINC_IOV_MAX_INIT;
+            if (max <= 0)
+              max = 16;
+            linc_iov_max = max;
+          }
+        G_UNLOCK (linc_iov_max);
+      }
+  }
+#else
+# define linc_iov_max_init()
+#endif
+
+static glong
+write_data (LINCConnection *cnx, QueuedWrite *qw)
 {
+	glong bytes_written = 0;
+
 	g_return_val_if_fail (cnx->status == LINC_CONNECTED,
 			      LINC_IO_FATAL_ERROR);
 
+	linc_iov_max_init ();
+
 	while ((qw->nvecs > 0) && (qw->vecs->iov_len > 0)) {
 		int n;
 
-		d_printf ("write_data %ld bytes to fd %d : %s\n",
-			  calc_size (qw->vecs, qw->nvecs),
-			  cnx->priv->fd, queue_write ? "from queue" : "direct");
+		d_printf ("write_data %ld bytes to fd %d - ",
+			  calc_size (qw->vecs, qw->nvecs), cnx->priv->fd);
 
 #ifdef LINC_SSL_SUPPORT
 		if (cnx->options & LINC_CONNECTION_SSL)
@@ -497,7 +541,9 @@ write_data (LINCConnection *cnx, QueuedWrite *qw, gboolean queue_write)
 		else
 #endif
 			n = writev (cnx->priv->fd, qw->vecs,
-				    MIN (qw->nvecs, WRITEV_IOVEC_LIMIT));
+				    MIN (qw->nvecs, LINC_IOV_MAX));
+
+		d_printf ("wrote %d bytes\n", n);
 
 		if (n < 0) {
 #ifdef LINC_SSL_SUPPORT
@@ -532,9 +578,9 @@ write_data (LINCConnection *cnx, QueuedWrite *qw, gboolean queue_write)
 			return LINC_IO_FATAL_ERROR;
 
 		else {
-			glong size = n;
+			bytes_written += n;
 
-			while (n >= qw->vecs->iov_len) {
+			while (qw->nvecs > 0 && n >= qw->vecs->iov_len) {
 				n -= qw->vecs->iov_len;
 				qw->nvecs--;
 				qw->vecs++;
@@ -544,13 +590,10 @@ write_data (LINCConnection *cnx, QueuedWrite *qw, gboolean queue_write)
 				qw->vecs->iov_len  -= n;
 				qw->vecs->iov_base += n;
 			}
-
-			if (queue_write)
-				queue_signal (cnx, -size);
 		}
 	}
 
-	return LINC_IO_OK;
+	return bytes_written;
 }
 
 /**
@@ -574,8 +617,8 @@ linc_connection_writev (LINCConnection       *cnx,
 			int                   nvecs,
 			const LINCWriteOpts  *opt_write_opts)
 {
-	QueuedWrite     qw;
-	LINCIOStatus status;
+	QueuedWrite qw;
+	glong       status;
 
 	/* FIXME: need an option to turn this off ? */
 	if (cnx->options & LINC_CONNECTION_NONBLOCKING) {
@@ -597,7 +640,7 @@ linc_connection_writev (LINCConnection       *cnx,
 	qw.nvecs = nvecs;
 
  continue_write:
-	status = write_data (cnx, &qw, FALSE);
+	status = write_data (cnx, &qw);
 
 	if (status == LINC_IO_QUEUED_DATA) {
 		/* Queue data & listen for buffer space */
@@ -611,7 +654,9 @@ linc_connection_writev (LINCConnection       *cnx,
 			linc_main_iteration (TRUE);
 			goto continue_write;
 		}
-	}
+
+	} else if (status >= LINC_IO_OK)
+		status = LINC_IO_OK;
 
 	return status;
 }
@@ -798,18 +843,20 @@ linc_connection_io_handler (GIOChannel  *gioc,
 		d_printf ("IO Out - buffer space free ...\n");
 
 		if (cnx->priv->write_queue) {
+			glong        status;
 			QueuedWrite *qw = cnx->priv->write_queue->data;
-			LINCIOStatus status;
 
-			status = write_data (cnx, qw, TRUE);
+			status = write_data (cnx, qw);
 
-			d_printf ("Wrote queue %d on fd %d\n", status, cnx->priv->fd);
+			d_printf ("Wrote queue %ld on fd %d\n", status, cnx->priv->fd);
 
-			if (status == LINC_IO_OK) {
+			if (status >= LINC_IO_OK) {
 				cnx->priv->write_queue = g_list_delete_link (
 					cnx->priv->write_queue, cnx->priv->write_queue);
 				queued_write_free (qw);
 
+				queue_signal (cnx, -status);
+
 				done_writes = (cnx->priv->write_queue == NULL);
 
 			} else if (status == LINC_IO_FATAL_ERROR) {
